<?php
	require_once("/usr/lib/png.mx/PNG_LoanObjects.lib.php");
	require_once("/usr/lib/png.mx/aws.phar");
	
	use Aws\Glacier\GlacierClient;
	use Guzzle\Http\EntityBody;
	use Aws\Common\Hash\TreeHash;
	
	class Glacier_Upload extends uObject {
		
		protected $partSize		= null; 			// numero  de megas para la parte
		protected $Checksums		= Array();		// array de checksums de multipart
		protected $masterChecksum	= null;			// master checksum para completar el upload
		protected $uploadID		= null;			// ID de upload
		protected $Location		= null;			// Location de upload
		protected $Vault		= null;			// vault para guardar
		protected $archiveSize		= 0;			// tamaño total del archive
		protected $Path			= null;			// path del archivo
		protected $Service		= null;			//objeto de cliente del Glacier
		protected $Desc			= null;
		protected $AccessKey		= null;
		protected $Secret 		= null;
		protected $Region		= 'us-east-1';
		protected $Credenciales		= null;
		
		var $Debugger2Log 		= false;
		//var $DEBUG 			= 3;
		
		function __construct($credenciales = null, $extras = null) {
			
			if(is_array($credenciales)) { $this->setCredenciales($credenciales); }
			
			if(is_array($extras)) { $this->__setInstanceData($extras); }
			
			return $this;		
			
		}
		
		function __setInstanceData($data) {
			#uDOC:TITLE __setInstanceData
			#uDOC:DESCR Pone datos de instancia 
			
			if(! is_array($data)) { return $this->setError("Imposible asignar datos, no recibi array"); }
			if(isset($data['uploadId'])) 		{ $this->uploadID = $data['uploadId']; } else { $this->setWarning("No tengo uploadid!"); }
			if(isset($data['location'])) 		{ $this->Location = $data['location']; } else { $this->setWarning("No tengo location!"); }
			if(isset($data['vaultName'])) 		{ $this->Vault 	= $data['vaultName']; } else { $this->setWarning("No tengo vaultName!"); }
			if(isset($data['partSize'])) 		{ $this->partSize = $data['partSize']; } else {  $this->setWarning("No tengo partSize!"); }
			if(isset($data['path']))		{ $this->Path = $data['path']; } else { $this->setWarning("No tengo path!"); }
			if(isset($data['archiveDescription']))	{ $this->Desc = $data['archiveDescription']; } else { $this->setWarning("No tengo desc!"); }
			
			return true;
		}
		
		function Credenciales() {
			#uDOC:TITLE Credenciales
			#uDOC:DESCR Regresa credenciales
			
			if($this->Credenciales === null) {
				$this->Credenciales['key'] 	= $this->AccessKey();
				$this->Credenciales['secret']	= $this->Secret();
				$this->Credenciales['region']	= $this->Region();
			}
			
			return $this->Credenciales;
			
		}
		
		function setCredenciales($credenciales) {
			#uDOC:TITLE setCredenciales
			#uDOC:DESCR Pone credenciales para usar cliente de aws
			
			if(! is_array($credenciales)) { return $this->setError("Imposible poner credenciales sin credenciales"); } 
			
			if(! isset($credenciales['key'])) { return $this->setError("Imposible crear cliente de glaciar sin llave"); }
			if(! isset($credenciales['secret'])) { return $this->setError("Imposible crear cliente de glaciar sin secret"); }
			if(! isset($credenciales['region'])) { return $this->setError("Imposible crear cliente de glaciar sin region"); }
			
			$this->AccessKey = $credenciales['key'];
			$this->Secret	 = $credenciales['secret'];
			$this->Region	 = $credenciales['region'];
			
			return true;
			
		}
		
		function AccessKey() {
			#uDOC:TITLE AccessKey
			#uDOC:DESCR Regresa Access key para aws
			
			return $this->AccessKey;
		}
		
		function Secret() {
			#uDOC:TITLE Secret
			#uDOC:DESCR Regresa secret para aws
			
			return $this->Secret;
		}
		
		function Region() {
			#uDOC:TITLE Region
			#uDOC:DESCR Regresa region para aws
			
			return $this->Region;
		}
		
		function Desc() {
			
			if($this->Desc === null) {  
				$this->Desc = $this->Vault() . "-" . date("Y-m-d") . "-" . date("H:i:s");  
			}
			
			return $this->Desc;
		}
		
		function Service() {
			#uDOC:TITLE Service
			#uDOC:DESCR Regresa Servicio de Glaciar del AWS
			
			if(! is_array($this->Credenciales())) { return $this->setError("Imposible regresar servicio sin credenciales"); }
			
			
			if($this->Service === null) {
				try {
					$this->Service = GlacierClient::factory(array(
					    'key'    => $this->AccessKey(),
					    'secret' => $this->Secret(),
					    'region' => $this->Region()
					));
					
				} catch (Exception $e) {
				    
					return $this->setError($e->getMessage());
				}
			}
			
			return $this->Service;
		}
		
		
		
		function archiveSize() {
			#uDOC:TITLE archiveSize
			#uDOC:DESCR Regresa tamanño de archive
			
			return $this->archiveSize;
		}
		
		function partSize() {
			#uDOC:TITLE partSize
			#uDOC:DESCR Tamaño de la parte para subir
			
			$bytes = $this->partSize * 1048576;
			
			return $bytes;
			
		}
		
		function Checksums() {
			#uDOC:TITLE Checksums
			#uDOC:DESCR Regresa coleccion de cheksums de multipart
			
			return $this->Checksums; 
		}
		
		function Vault() {
			#uDOC:TITLE Vault
			#uDOC:DESCR Regresa vault para usar 
			
			return $this->Vault;
			
		}
		
		function uploadID() {
			#uDOC:TITLE uploadID
			#uDOC:DESCR Id de upload
			
			return $this->uploadID;
			
		}
		
		function Location() {
			#uDOC:TITLE Location
			#uDOC:DESCR location de upload
			
			return $this->Location;
			
		}
		
		function Path() {
			#uDOC TITLE Path
			#uDOC:DESCR Regresa path del archivo a subir
			
			return $this->Path;
		}
		
		function setVault($vault) {
			#uDOC:TITLE setVault
			#uDOC:DESCR Pone vault a usar para la subida
			
			if(! $vault) { return $this->setError("Imposible poner vault sin vault"); }
 			
			$this->Vault = $vault;
		}
		
		function setPartSize($size) {
			#uDOC:TITLE setPartSize
			#uDOC:DESCR Set tamaño de la parte para subir al glaciar
			
			if(! is_int($size)) { return $this->setError("Numero invalido para tamaño de parte");  } 
			
			$this->partSize = $size;
			
			return true;
			
		}
		
		function setPath($path) {
			#uDOC:TITLE setPath
			#uDOC:DESCR Pone path del archivo
			
			if(! is_file($path)) { return $this->setError("Path de archivo invalido"); }
			
			$this->Path = $path;
			
			return true;
		}
		
		
		function setUploadID($upid) {
			#uDOC:TITLE setUploadID
			#uDOC:DESCR Sube un archivo por partes
			if(! $upid) { return $this->setError("Imposible poner upload id sin id "); }
			
			$this->uploadID = $upid;
			
			return true;
		}
		
		function setLocation($location) {
			#uDOC:TITLE setLocation
			#uDOC:DESCR Pone location de un upload iniciada
			if(! $location) { return $this->setError("Imposible poner location sin location"); }
			
			$this->Location = $location;
			
			return true;
		}
		
		
		function Stream() {
			#uDOC:TITLE Stream
			#uDOC:DESCR Bandera de tipo de upload
			
			//antes que nada
			if(! $this->Path()) { return $this->setError("Imposible subir archivo sin path"); }
			if(! $this->Vault()) { return $this->setError("Imposible subir archivo sin vault"); }
			
			
			if(! is_file($this->Path())) { return $this->setError("Imposible subir archivo no existe");  }
			
			
			
			if(! $this->uploadID()) {
				
				if(! $res = $this->initiateMultipartUpload()) { return $this->setError("No pude iniciar el upload"); }
			
				$this->setUploadID($res->get('uploadId'));
				$this->setLocation($res->get('location'));
			}
			
			
			//Aqui puedes decidir si hacerlo por partes o streameado 
			return 	$this->streamParts();
				
		}
		
		function streamParts() {
			#uDOC:TITLE streamParts
			#uDOC:DESCR Streamea partes de multipart
			
			if(! $this->partSize()) { return $this->setError("Imposible subir archivo sin partSize"); }
			
			$handle = popen("/bin/tar -cv  -T " . $this->Path(), "r");
			
			$content = '';
			$checksums  = Array();
			$currentPosition = 0;
			
			gc_enable(); // Enable Garbage Collector
			//var_dump(gc_enabled()); // true
			
			while (!feof($handle)) {
				
				//$content = fread($handle, $this->bytesPartSize());
				$content = stream_get_contents($handle, $this->partSize());
				
				$readlen = strlen($content); 
					
				$this->archiveSize = $this->archiveSize() + $readlen;
				
				$endrange = ($currentPosition + $readlen) - 1;
				
				$range = "bytes " .$currentPosition . "-" . $endrange ."/*"; 
				
				$currentPosition = $currentPosition + $this->partSize();
				
				// aqui se puede probar si falla o algo en la subida de la parte e intentarlo de nuebvo despues
				
				do {
					if(! $res = $this->uploadMultipartPart($content, $range)) {  $this->setWarning("No pude subir la parte lo vuelvo a intentar.");  }
				
				} while (! $res);
				
				$this->debuggerShow("van : " . $range);
				$this->debuggerShow("RAM : " . memory_get_usage(true));
				$this->debuggerShow("Garbage collector: " . gc_collect_cycles());
				//var_dump(gc_collect_cycles()); // # of elements cleaned up
				
			}
			
			fclose($handle);
			//quita recolector de basura
			gc_disable();
			
			return $this->completeMultipartUpload();
			
			
		}
		
		function Upload() {
		
			//antes que nada
			if(! $this->Path()) { return $this->setError("Imposible subir archivo sin path"); }
			
			if(! is_file($this->Path())) { return $this->setError("Imposible subir archivo no existe");  }
			
			if(! $this->uploadID()) {
				
				if(! $res = $this->initiateMultipartUpload()) { return $this->setError("No pude iniciar el upload"); }
			
				$this->setUploadID($res->get('uploadId'));
				$this->setLocation($res->get('location'));
			}
			
			
			//Aqui puedes decidir si hacerlo por partes o streameado 
			return 	$this->uploadParts();
			
		}
		
		function uploadParts() {
			#uDOC:TITLE uploadParts
			#uDOC:DESCR Sube partes de multipart
			
			//FIXME: Implementar upload varios a la vez cuando sea un archivo escrito al disco   
			
			return $this->setError("falta implentar esta funcion");
			
		}
		
		function initiateMultipartUpload() {
			#uDOC:TITLE initiateMultipartUpload
			#uDOC:DESCR Inicia subida de archivos por partes para archivos mayores a 100MB
			
			if(! $this->Vault()) { return $this->setError("Imposible subir archivo sin vault"); }
			if(! $this->partSize()) { return $this->setError("Imposible subir archivo sin partsize"); }
			
			try {
				$result = $this->Service()->initiateMultipartUpload(array(
				    'vaultName'          => $this->Vault(),
				    'archiveDescription' => $this->Desc(),
				    'partSize'           => $this->partSize() 
				));
				
			} catch (Exception $e) {
			    return $this->setError($e->getMessage());
			}
			
			return $result;
			
		}
		
		function uploadMultipartPart($content, $range) {
			#uDOC:TITLE uploadMultipartPart
			#uDOC:DESCR Sube parte de un multipartupload
				
			if(! $content) { return $this->setError("Imposible subir multiparte sin content"); }
			if(! $range)   { return $this->setError("Imposible subir multiparte sin range"); }
			if(! $this->Vault()) { return $this->setError("Imposible subir multiparte sin vault"); }
			if(! $this->uploadID()) { return $this->setError("Imposible subir multiparte sin uploadID"); }
			
			try {
				$result = $this->Service()->uploadMultipartPart(array(
				    'vaultName'         => $this->Vault(),
				    'uploadId' 		=> $this->uploadID(),
				    'range'		=> $range,
				    'body'		=> $content  
				));
				
					
				
			} catch (Exception $e) {
			    return $this->setError($e->getMessage());
			}
			
			$this->Checksums[] = $result->get('checksum');
			
			return true;
			
		}
		
		function completeMultipartUpload() {
			#uDOC:TITLE uploadMultipartPar
			#uDOC:DESCR AVisa a Glaciar que el multiupload a terminado y puede ensamblar las partes
			
			if(! $this->Vault()) 		{ return $this->setError("Imposible completar archivo sin vault"); }
			if(! $this->uploadID()) 	{ return $this->setError("Imposible completar archivo sin uploadID"); }
			if(! $this->archiveSize()) 	{ return $this->setError("Imposible completar archivo sin archivSize"); }
			if(! $this->masterChecksum()) 	{ return $this->setError("Imposible completar archivo sin masterChecksum"); }
			
			try {
				
				$result = $this->Service()->completeMultipartUpload(array(
				    'vaultName' 	=> $this->Vault(),
				    'uploadId' 		=> $this->uploadID(),
				    'archiveSize'	=> $this->archiveSize(),
				    'checksum'		=> $this->masterChecksum()
				));
			
			} catch (Exception $e) {
			    return $this->setError($e->getMessage());
			}
				
			return $result;
			
		}
		
		function masterChecksum() {
			#uDOC:TITLE masterChecksum
			#uDOC:DESCR calcula chec
			
			if(! is_array($this->Checksums())) {  return $this->setError("Imposible sacar maaaaster checksum sin checksums"); }
			
			if ($this->masterChecksum === null) {
			    
				$this->masterChecksum = TreeHash::fromChecksums($this->Checksums())->getHash();
			}
		
			return $this->masterChecksum;
		}
		
		function listParts() {
			#uDOC:TITLE listParts
			#uDOC:DESCR lista partes de un multipartupload que ya esta en glaciar
			
			if(! $this->uploadID()) { return $this->setError("Imposible listar partes sin uploadid"); }
			if(! $this->Vault()) { return $this->setError("Imposible listar partes sin vault"); }
			
			try {
				
				$result = $this->Service()->listParts(array(
				    'vaultName' 	=> $this->Vault(),
				    'uploadId' 		=> $this->uploadID(),
				));
				
			
			} catch (Exception $e) {
			    return $this->setError($e->getMessage());
			}
				
			return $result;
		}
		
		
	}
?>
