<?php
	require_once("/usr/lib/png.mx/PNG_LoanObjects.lib.php");
	require_once("/usr/lib/png.mx/PNG_Client_Server.class.php");
	
	class PNG_Client_Customer extends uObject {
		
		
		protected $APIKey		= null;			// Api key del customer
		protected $UUID			= null;			// UUID del cliente en el sistema
		protected $Nombre		= null;			// Nombre humano del cliente
		protected $XMLRPC		= null;			// Cliente de XMLRPC uServers
		
		function _loadMetaData() {
			#uDOC:TITLE _loadMetaData
			#uDOC:DESCR Carga el archivo de metadatos del server
			
			if(! is_file($this->metaDataFileName())) { return $this->setError("Imposible sacar datos de customer, no existe archivo de configuracion "); }
			if(! is_readable($this->metaDataFileName())) { return $this->setError("Imposible sacar metadatos del customer, no puedo leer el file"); }
			
			
			$raw_data = file_get_contents($this->metaDataFileName());
			if(! $raw_data) { return $this->setError("Imposible sacar metadatos del server, no pude leer"); }
			
			$metadata = @unserialize($raw_data);
			if(is_array($metadata)) { return $this->_setInstanceData($metadata); }
			else { return $this->setError("Error al interpretar file de metadatos"); }
 
		}
		
		function _setInstanceData($metadata) {
			#uDOC:TITLE _setInstanceData
			#uDOC:DESCR Asigna los datos de la instancia
			
			if(! is_array($metadata)) { return $this->setError("Imposible asignar datos de instancia, no recibi array"); }
			
			if(isset($metadata['uuid'])) { $this->UUID = $metadata['uuid']; }
			return true;
		}
		
		
		function metaDataFileName() {
			#uDOC:TITLE metaDataFileName
			#uDOC:DESCR Regresa el filename del archivo de configuracion
			
			return "/etc/png.mx/customer.data";
		}
		
		
		function APIKey() {
			#uDOC:TITLE APIKey
			#uDOC:DESCR Regresa la key del api del customer
			
			return $this->APIKey;
		}
		
		function UUID() {
			#uDOC:TITLE UUID
			#uDOC:DESCR Regresa el UUID del customer en png.mx
			
			if($this->UUID === null) { $this->_loadMetaData(); }
			return $this->UUID;
			
		}
		
		function registerServer() {
			#uDOC:TITLE registerServer
			#uDOC:DESCR Registra el server en png.mx para empezarlo a usar
			
			
			// Dummy server para sacar el path del file de configuracion:
			$server = new PNG_Client_Server();
			$path = $server->metaDataFileName();
			// Checa si ya esta registrado:
			if(is_file($path)) { return $this->setError("Imposible registrar server, ya existe el file de configuracion"); }
			
			$server_data = $server->serverData();
			
			// Llama al RPC para registrar la maquina:
			$data = $this->Call("png.customer.registerServer", $server_data);
			
			// Si recibimos el array de datos, hay que instalarlo:
			if(is_array($data)) {
				if(! count($data)) { return $this->setError("Imposible registrar server, recibi array de info vacio!"); }
				$data_string = serialize($data);
				
				// Pon los datos en el file:
				file_put_contents($path, $data_string);
				
				// Un poco de debug:
				$this->debuggerShow("Server registrado exitosamente!");
				$this->debuggerShow("UUID: " . $data['uuid'], 2);
				
				return true;
			} else {
				return false;
			}
			
		}
		
		function XMLRPC() {
			#uDOC:TITLE XMLRPC
			#uDOC:DESCR Regresa un cliente reusable de XMLRPC
			
			if(! is_object($this->XMLRPC)) {
				if(! $this->UUID()) { return $this->setError("Imposible generar mi cliente de XMLRPC, no tengo mi UUID"); }
				$this->XMLRPC = new uXMLRPC_Client("http://api.png.mx/customer/" . $this->UUID());
			}
			
			return $this->XMLRPC;
			
		}
		
		function Call($method, $args = null) {
			#uDOC:TITLE Call
			#uDOC:DESCR Haz una llamada al XMLRPC y regresa el resultado
			
			if(! $method) { return $this->setError("Imposible llamar remoto: no recibi metodo"); }
			if(! is_object($this->XMLRPC())) { return $this->setError("Impsible llamar metodo remoto, no tengo cliente de XMLRPC"); }
			
			$data = $this->XMLRPC()->Call($method, $args);
			if(! is_array($data)) { return $this->setError("Error de XMLRPC: " . $this->XMLRPC()->Error()); }
			if((isset($data['error'])) && ($data['error'])) { return $this->setError("Error remoto: " . $data['error']); }
			if((isset($data['success'])) && (isset($data['success'])) ) { return true; } 
			
			return $data;
			
		} 
	}
	
	
	
?>
