<?php

	require_once("/usr/lib/png.mx/PNG_LoanObjects.lib.php");
	
	class PNG_Client_Server extends uObject {
		
		protected $XMLRPC			= null;			// Cliente de XMLRPC uServers
		protected $UUID				= null;			// UUID del server en el sistema de PNG
		protected $APIKey			= null;			// Key del API particular de este server
		
		function APIKey() {
			if($this->APIKey === null) { $this->_loadMetaData(); }
			return $this->APIKey;
		}
		
		function UUID() {
			#uDOC:TITLE UUID
			#uDOC:DESCR Regresa el UUID del server en el sistema
			
			if($this->UUID === null) { $this->_loadMetaData(); }
			return $this->UUID;
			
		}
		
		function _loadMetaData() {
			#uDOC:TITLE _loadMetaData
			#uDOC:DESCR Carga el archivo de metadatos del server
			
			if(! is_file($this->metaDataFileName())) { return $this->setError("Imposible sacar datos de server, no existe archivo de configuracion (no esta registrado?) "); }
			if(! is_readable($this->metaDataFileName())) { return $this->setError("Imposible sacar metadatos del server, no puedo leer el file"); }
			
			
			$raw_data = file_get_contents($this->metaDataFileName());
			if(! $raw_data) { return $this->setError("Imposible sacar metadatos del server, no pude leer"); }
			
			$metadata = @unserialize($raw_data);
			if(is_array($metadata)) { return $this->_setInstanceData($metadata); }
			else { return $this->setError("Error al interpretar file de metadatos"); }
 
		}
		
		function _setInstanceData($metadata) {
			#uDOC:TITLE _setInstanceData
			#uDOC:DESCR Asigna los datos de la instancia
			
			if(! is_array($metadata)) { return $this->setError("Imposible asignar datos de instancia, no recibi array"); }
			
			
			if(isset($metadata['apikey']))		{ $this->APIKey = $metadata['apikey']; }
			if(isset($metadata['uuid']))		{ $this->UUID = $metadata['uuid']; }
			return true;
		}
		
		
		function metaDataFileName() {
			#uDOC:TITLE metaDataFileName
			#uDOC:DESCR Regresa el filename del archivo de configuracion
			
			return "/etc/png.mx/server.data";
		}
		
		function updatesList() {
			#uDOC:TITLE updatesList
			#uDOC:DESCR Carga una lista de updates desde la salida del yum
			
			if(! is_executable("/usr/bin/yum")) { return $this->setError("Imposible sacar mi lista de updates, no puedo ejecutar el yum"); }
			// FIXME: If (! soy root)
	
			// Saca la lisada del yum:
			$this->debuggerShow("Iniciando ejecucion de yum", 2);
			$raw = `/usr/bin/yum check-update`;
			if(! $raw) { return $this->setError("Imposible sacar lista de updates, no recibi nada del yum"); }
			
			$lineas = explode("\n", $raw);
			$updates = Array();
			
			// Cicla todas las lineas hasta encontrar donde esta:
			foreach($lineas as $linea) {
				$update = Array(); 
				
				// Parsea la linea de updates, a ver si es de i386 o 64bits
				if(preg_match('/^(\S+)\.(i\d86)\s+(\S+)/', $linea, $m))		{ $update["package"] = $m[1]; $update['version'] = $m[3]; $update['arch'] = $m[2]; }
				if(preg_match('/^(\S+)\.(x86_64)\s+(\S+)+/', $linea, $m))	{ $update["package"] = $m[1]; $update['version'] = $m[3];  $update['arch'] = $m[2]; }
				if(preg_match('/^(\S+)\.noarch\s+(\S+)+/', $linea, $m))		{ $update["package"] = $m[1]; $update['version'] = $m[2];  $update['arch'] = "noarch"; }
				
				// Si es un update, agrega al update
				if(count($update)) {
					$key = $update['package'] . "-" . $update['version'] . "-" . $update["arch"];
					$updates[$key] = $update;
				}
				
			}
			
			// Regresala:
			return $updates;
			
		}
		
		function registerUpdates() {
			#uDOC:TITLE registerUpdates
			#uDOC:DESCR Publica la lista de updates al server central
			
			
			// if(! is_object($this->XMLRPC())) { return false; }
			$updates = $this->updatesList();
			if(! is_array($updates)) { return $this->setError("Imposible publicar updates, no tengo la lista de updates"); }
			
			$res = $this->Call("png.server.registerUpdates", Array($updates));
			
			if($res) { $this->debuggerShow(count($updates) . " updates pendientes registrados"); return true; }
			else { return false; }
			
		}
		
		function XMLRPC() {
			#uDOC:TITLE XMLRPC
			#uDOC:DESCR Regresa un cliente reusable de XMLRPC
			
			if(! is_object($this->XMLRPC)) {
				if(! $this->UUID()) { return $this->setError("Imposible generar mi cliente de XMLRPC, no tengo mi UUID"); }
				$this->XMLRPC = new uXMLRPC_Client("http://api.png.mx/server/" . $this->UUID());
				$this->XMLRPC->Debug($this->Debug());
			}
			
			return $this->XMLRPC;
			
		}
		
		function Call($method, $args = null) {
			#uDOC:TITLE Call
			#uDOC:DESCR Haz una llamada al XMLRPC y regresa el resultado
			
			if(! $method) { return $this->setError("Imposible llamar remoto: no recibi metodo"); }
			if(! is_object($this->XMLRPC())) { return $this->setError("Impsible llamar metodo remoto, no tengo cliente de XMLRPC"); }
			// $this->XMLRPC()->Debug(5);
			$data = $this->XMLRPC()->Call($method, $args);
			
			if(! is_array($data)) { return $this->setError("Error de XMLRPC: " . $this->XMLRPC()->Error()); }
			
			if((isset($data['error'])) && ($data['error'])) { return $this->setError("Error remoto: " . $data['error']); }
			
			// Interpreta el true ??
			return $data;
			
		}
		
		
		function filesystemData() {
			#uDOC:TITLE filesystemData
			#uDOC:DESCR Regresa informacion sobre los filesystems de la maquina
			
			if(! is_executable("/bin/df")) { return $this->setError("Impsible ejecutar df!"); }
			
			// Saca las lineas del df
			$raw_data = `/bin/df -k`;
			if(! $raw_data) { return $this->setError("Imposible obtener datos de filesystems"); }
			
			// Parte en lineas individuales:
			$lineas_raw = explode("\n", $raw_data);
			array_shift($lineas_raw);
			
			// Ahora hay que unir las que son continuaciones:
			$lineas = Array();
			$current_linea = '';
			foreach($lineas_raw as $linea) {
				
				// Si es un linea blanca, ignora
				if(preg_match('/^\s*$/', $linea)) { continue; }
				
				// Si empieza con espacio, es una continuacion de la anterior:
				elseif(preg_match('/^\s+/', $linea)) { $lineas[count($lineas) - 1] .= $linea; }
				
				// Si no es el inicio de una linea nueva
				else { $lineas[] = $linea; }
			}
			
			// Ahora vamos a interpretarlas, ejemplo:
			//ZARATUSTRA/CoC_Libs                       8103936   5969536   2134400  74% /var/www/html/proyectos
			$fsdata = Array();
			foreach($lineas as $linea) {
				trim($linea);
				$partes = preg_split("/\s+/", $linea);
				if(! count($partes) == 5) { $this->setError("Imposible interpretar esto: $linea"); continue; }
				
				$fsdata[$partes[5]] = Array(
					"mountpoint"	=> $partes[5],
					"device"		=> $partes[0],
					"size"			=> $partes[1],
					"used"			=> $partes[2],
					"available"		=> $partes[3],
					
				);
				
			}
			
			return $fsdata;
			
			
		}
		
		
		function hostnameData() {
			#uDOC:TITLE hostnameData
			#uDOC:DESCR Regresa los datos del hostname de la maquina en un array
			
			if(! is_executable("/bin/hostname")) { return $this->setError("Imposible ejecutar /bin/hostname !"); }
			
			// Saca el nombre:
			$full_hostname = `/bin/hostname`;
			if(! $full_hostname) { return $this->setError("Imposible obtener el nombre de la maquina!"); }
			$full_hostname = trim($full_hostname);
			
			// Array para regresar
			$server_data = Array();
			
			// Checa si el hostname trae domain:
			if(strpos($full_hostname, ".") !== false) {
				$partes = explode('.', $full_hostname);
				$server_data['hostname'] = $partes[0];
				unset($partes[0]);
				$server_data['domain'] = join('.', $partes);
				
			} else {
				$server_data['hostname'] = $full_hostname;
				$server_data['domain'] = '';
			}
			
			return $server_data;
			
		}
		
		function soyRoot() {
			#uDOC:TITLE soyRoot
			#uDOC:DESCR Regresa si el proceso esta corriendo como root o no
			
			return true;
			
			$uid = posix_getuid();
			// print "UID es: " . $uid . "\n";
			if($uid === 0) { return true; }
			else { return false; }
			
		}
			

		
		function latestKernel() {
			#uDOC:TITLE latestKernel
			#uDOC:DESCR Regresa la version mas reciente del kernel instalada
			
			if(! is_executable("/bin/rpm")) { return $this->setError("Imposible sacar version del kernel, no puedo ejecutar RPM!"); }
			if(! is_executable("/bin/sort")) { return $this->setError("Imposible sacar ultimo kernel, no puedo encontrar el sort"); }
			if(! is_executable("/usr/bin/head")) { return $this->setError("Imposible sacar version del kernel, no puedo encontrar head"); }
			// Llama al sistem
			
			$ver = @`/bin/rpm -q kernel | /bin/sort -r | /usr/bin/head -n1`;
			if(! $ver) { return $this->setError("Imposible encontrar el kernel mas reciente!"); }
			
			$ver = trim($ver);
			$ver = str_replace("kernel-", "", $ver);
			return $ver;
		}
		
		
		function kernelVersion() {
			#uDOC:TITLE kernelVersion
			#uDOC:DESCR Regresa la version del kernel que esta corriendo
			
			if(! is_executable("/bin/uname")) { return $this->setError("Imposible encontrar uname !"); }
			
			$kernel = `/bin/uname -r`;
			$kernel = trim($kernel);
			
			return $kernel;
			
		}
		
		
		function loadAverages() {
			#uDOC:TITLE loadAverages
			#uDOC:DESCR Regresa un array con los load averages del sistema
			
			if(! is_executable("/usr/bin/uptime")) { return $this->setError("Imposible encontrar el uptime!"); }
			$uptime = `/usr/bin/uptime`;
			if(! $uptime) { return $this->setError("Imposible sacar averages del uptime !"); }
			if(preg_match('/(\d+\.\d+),\s+(\d+\.\d+),\s+(\d+\.\d+)\s*$/', $uptime, $m)) {
				return Array(
					"a1"	=> $m[1],
					"a5"	=> $m[2],
					"a15"	=> $m[3],
				);
			} else {
				return $this->setError("Imposible encontrar los promedios de load en la salida !");
			}
			
			
		}
		
		function serverData() {
			#uDOC:TITLE serverData
			#uDOC:DESCR Regresa un array con datos del server
			
			$server_data = Array();
			
			// Saca el nombre del host:
			$hostdata = $this->hostnameData();
			if(! is_array($hostdata)) { $this->setError("Imposible sacar hostname de servidor!"); }
			else { $server_data['hostname'] = $hostdata['hostname']; $server_data['domain'] = $hostdata['domain']; }
			
			// Mete la version del kernel:
			$kernel = $this->kernelVersion();
			if($kernel) { $server_data['kernel'] = $kernel; } else { $this->setError("Imposible encontrar la version del kernel"); }
			
			// Saca la version del kernel mas reciente:
			$lkernel = $this->latestKernel();
			if($lkernel) { $server_data['lkernel'] = $lkernel; } else { $this->setError("Imposible encontrar la version mas reciente del kernel"); }
			
			// Saca informacion de load:
			$load = $this->loadAverages();
			if($load) { $server_data['load'] = $load; } else { $this->setError("Imposible obtener los promedios de load"); }
			
			// Saca informacion de los filesystems:
			$fsdata = $this->filesystemData();
			if(! is_array($fsdata)) { $this->setError("Imposible sacar informacion de filesystems"); }
			else { $server_data['filesystems'] = $fsdata; }
			
			// Saca la lista de puertos abiertos:
			$ports = $this->openPorts();
			if(! is_array($ports)) { return $this->setError("Imposible sacar informacion de los puertos abiertos"); }
			else { $server_data['ports'] = $ports; }
			
			$cpuinfo = $this->cpuInfo();
			if(! is_array($cpuinfo)) { return $this->setError("Imposible sacar informacion del CPU de la maquina"); }
			else { $server_data['cpu'] = $cpuinfo; }
			
			// print "<pre>"; print_r($ports); print "</pre>";exit;
			return $server_data;
			
		}
		
		function cpuInfo() {
			#uDOC:TITLE cpuInfo
			#uDOC:DESCR Regresa un array con informacion de los CPUs de la maquina
			
			if((! is_file("/proc/cpuinfo")) || (! is_readable("/proc/cpuinfo"))) { return $this->setError("Imposible leer informacion de los CPUs de la maquina, no puedo leer archivo"); }
			$todo = file_get_contents("/proc/cpuinfo");
			
			// Datos que recopilamos:
			$num_cpus	= 0;
			$bogo_array	= Array();
			$vendor		= null;
			$model		= null;
			$mhz		= null;
			
			foreach(explode("\n", $todo) as $linea) {
				if(preg_match('/^processor\s+:\s+(\d+)/', $linea, $m)) {
					// if($num_cpus < $m[1]) { $num_cpus = intval($m[1]); }
					$num_cpus++;
				} elseif(preg_match('/^bogomips\s+:\s+(\d+\.\d+)/', $linea, $m)) {
					$bogo_array[] = $m[1];
				} elseif(preg_match('/^vendor_id\s+:\s+(.+)/', $linea, $m)) {
					$vendor = $m[1];
				} elseif(preg_match('/^model\s+name\s+:\s+(.+)/', $linea, $m)) {
					$model = $m[1];
				} elseif(preg_match('/^cpu\s+MHz\s+:\s+(.+)/', $linea, $m)) {
					$mhz = $m[1];
				}
			}
			
			// Checa si salio algo o no:
			if(! count($bogo_array)) {
				return $this->setError("Imposible calcular bogomips, no pude obtener ninguna lectura !");
			}
			
			
			//num_cpus
			
			/*if((is_executable("/bin/cat")) && (is_executable("/bin/grep")) && (is_executable("/usr/bin/wc"))) { 
				$num_cpus = `/bin/cat /proc/cpuinfo | /bin/grep processor | /usr/bin/wc -l`;
			}*/
			
			
			return Array(
				"cores"		=> $num_cpus,
				"bogomips"	=> array_sum($bogo_array) / count($bogo_array),
				"vendor"	=> $vendor,
				"model"		=> $model,
				"mhz"		=> $mhz,
			);
			
		}
		
		function publishServerData() {
			#uDOC:TITLE publishServerData
			#uDOC:DESCR Actualiza en el server la informacion del server
			
			$data = $this->serverData();
			// print "<pre>"; print_r($data); print "</pre>";exit;
			if(! is_array($data)) { return $this->setError("Imposible postear datos de server, no recibi array"); }
			
			return $this->Call("png.server.registerServerData", $data);
			
			
		}
		
		function openPorts() {
			#uDOC:TITLE openPorts
			#uDOC:DESCR Regresa un listado de los puertos estan abiertos en la maquina
			
			if(! $this->soyRoot()) { return $this->setError("Imposible determinar puertos, no soy root"); }
			
			// Verifica que tengamos los ejecutables:
			if(! is_executable("/bin/grep")) { return $this->setError("Imposible determinar puertos, no puedo ejecutar grep"); }
			if(! is_executable("/usr/sbin/lsof")) { return $this->setError("Imposible determinar puertos, no puedo ejecutar lsof"); }
			
			// Inicializa un array para regresar los puertos
			$ports = Array();
			
			// Saca la lista de puertos de TCP:
			$raw_data = `/usr/sbin/lsof -nP | /bin/grep LIST`;
			if(! $raw_data) { return $this->setError("Imposible obtener lectura de puertos abiertos"); }
			$lineas = explode("\n", $raw_data);
			
			foreach($lineas as $linea) {
				
				// Ejemplo de output para TCP:
				// httpd      1685 apache    3u     IPv6       4403                  TCP *:80 (LISTEN)
				if(preg_match('/^\s*(\S+).+TCP\s*(.+):(\d+)\s+\(LISTEN\)\s*$/', $linea, $m)){
					
					// Inicializa los arrays si no existen:
					if(! isset($ports['tcp'])) { $ports['tcp'] = Array(); }
					if(! isset($ports['tcp'][$m[2]])) { $ports['tcp'][$m[2]] = Array(); }
					if(! isset($ports['tcp'][$m[2]]["p".$m[3]])) { $ports['tcp'][$m[2]]["p".$m[3]] = Array(); }
					
					// Solo agrega el proceso si no esta ya listado:
					if(! in_array($m[1], $ports["tcp"][$m[2]]["p".$m[3]])) {
						$ports["tcp"][$m[2]]["p".$m[3]][] = $m[1];
					}
				}
			}
			
			// Saca la lista de puertos abiertos de UDP:
			$raw_data = `/usr/sbin/lsof  -nP | /bin/grep -P 'UDP\s+([\*0-9\.]+):'`;
			$lineas = explode("\n", $raw_data);
			
			foreach($lineas as $linea) {
				// Ejemplo de las lineas de UDP:
				// python     2878    urbano   47u     IPv4      19989       0t0        UDP 200.36.127.83:plysrv-https
				if(preg_match('/^\s*(\S+).+UDP\s+([\*0-9\.]+):(\d+)/', $linea, $m)) {
					// Inicializa los arrays:
					if( isset($ports['udp'])) { $ports['udp'] = Array(); }
					if(! isset($ports['udp'][$m[2]])) { $ports['udp'][$m[2]] = Array(); }
					if(! isset($ports['udp'][$m[2]]["p".$m[3]])) { $ports['udp'][$m[2]]["p".$m[3]] = Array(); }
					
					// Agrega el proceso si no esta ya:
					if(! in_array($m[1], $ports['udp'][$m[2]]["p".$m[3]])) {
						$ports['udp'][$m[2]]["p".$m[3]][] = $m[1];
					}
				}
				
			}
			
			// Bueno, por ahora regresamos la estructura de datos de esa forma:
			return $ports;
			
		}
		
		
		function fetchFSErrors() {
			#uDOC:TITLE fetchFSErrors
			#uDOC:DESCR busca errores del filesystem 
			
			if((is_executable("/bin/dmesg")) && (is_executable("/bin/grep")) ) { 
				$err = `dmesg | grep EXT | grep rror`;
				
				
			}
		}
		
		
		function pendingUpdates() {
			#uDOC:TITLE pendingUpdates
			#uDOC:DESCR Regresa la lista de updates pendientes por poner
			
			$updates = $this->Call("png.server.pendingUpdates");
			if(! is_array($updates)) { return false; }
			else { return $updates; }
			
		}
		
		function applyPendingUpdates() {
			#uDOC:TITLE applyPendingUpdates
			#uDOC:DESCR Aplica las actualizaciones que estan pendientes de entrar
			
			if(! is_executable("/usr/bin/yum")) { return $this->setError("Imposible aplicar updates, no encuentro a yum"); }
			
			$updates = $this->pendingUpdates();
			if(! is_array($updates)) { return $this->setError("Imposible obtener lista de updates!"); }
			if(! count($updates)) {
				$this->debuggerShow("No hay updates pendientes por aplicar");
				return true;
			}
			
			// Ahora si, mete todo lo pendiente
			foreach($updates as $update_data) {
				if(! $update_data['package']) { $this->setError("Updates sin paquete!"); continue; }
				
				// Escapa el nombre del paquete para echarlo al shell:
				$p = escapeshellarg($update_data['package']);
				$this->debuggerShow("Aplicando update $p");
				
				`/usr/bin/yum -y update $p`;
			}
			
			// Al final dale registerupdates para que el API sepa si algo falto:
			$this->registerUpdates();
			return true;
			
		}
		
		function Alerts() {
			#uDOC:TITLE Alerts
			#uDOC:DESCR Regresa las alertas registradas del server
			
			return $this->Call("png.server.listAlerts");
			
		}
		
		
		function listAlertsReminder() {
			#uDOC:TITLE listAlertsReminder
			#uDOC:DESCR Regresa las alertas que tegan reminder
			
			
			return $this->Call("png.server.listAlertsReminder");
			
		}
		
		function updateAlertReminder($uuid, $extras = null) {
			#uDOC:TITLE updateAlertReminder
			#uDOC:DESCR actuliza reminder de la alerta
			
			if(! $uuid) { return $this->setError("Imposible poner reminder sin uuid"); }
			return $this->Call("png.server.updateAlertReminder", Array($uuid,  $extras));
			
		}
		
		function setAlertReminder($uuid, $tipo ,$extras = null) {
			#uDOC:TITLE setAlertReminder
			#uDOC:DESCR pone recordatorio de la alerta
			
			
			
			if(! $uuid) { return $this->setError("Imposible poner reminder sin uuid"); }
			if(! $tipo) { return $this->setError("Imposible poner reminder sin tipo"); }
			return $this->Call("png.server.setAlertReminder", Array($uuid, $tipo, $extras));
			
			
		}
		
		function clearReminder($uuid) {
			#uDOC:TITLE clearReminder
			#uDOC:DESCR quita reminder de la alerta
			
			if(! $uuid) { return $this->setError("Imposible cancelar reminder de alerta, no recibi UUID"); }
			
			return  $this->Call("png.server.clearReminder", Array($uuid));
			
		}
		
		function registerAlert($message, $extras = null) {
			#uDOC:TITLE registerAlert
			#uDOC:DESCR Fija una alerta del server por el API
			
			if(! $message) { return $this->setError("Imposible regitrar alerta, no recibi mensaje"); }
			return $this->Call("png.server.registerAlert", Array($message, $extras));
		}
		
		function clearAlertCondition($uuid) {
			#uDOC:TITLE clearAlertCondition
			#uDOC:DESCR Cancela la condicion de alerta de una alerta
			
			if(! $uuid) { return $this->setError("Imposible cancelar condicion de alerta, no recibi UUID"); }
			
			return  $this->Call("png.server.clearAlertCondition", Array($uuid));
	
		}
		
		function dismissAlert($uuid) {
			#uDOC:TITLE dismissAlert
			#uDOC:DESCR Marca una alerta como leida para que ya no salga en la lista 
			
			if(! $uuid) { return $this->setError("Imposible dismisear alerta, no recibi UUID para procesar "); }
			
			return  $this->Call("png.server.dismissAlert", Array($uuid));
	
		}
		
		
		function processBackupJob() {
			#uDOC:TITLE processBackupJob
			#uDOC:DESCR Procesa job de backup
			
			return $this->Call("png.server.backupInfo");
			
		}
		
		function notifyBackupJob($res) {
			#uDOC:TITLE processBackupJob
			#uDOC:DESCR Procesa job de backup
			if(! $res) {  return $this->setError("Imposible notificar resultado de backup sin resultado"); }
			
			return $this->Call("png.server.notifyBackupJob",  Array($res));
			
		}
		
	}

?>
